#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include "serial.h"
#include "simulator.h"

static int serial_fd;
static bool serial_initialised = false;

extern int g_argc;
extern char ** g_argv;
static FILE *gcode_fd;

static int kbhit()
{
  volatile int *rx_ptr = (int *)stdin->drvarg[0];
  return (rx_ptr[0] != rx_ptr[1]);
}

static void makeraw()
{
  setvbuf(stdin, NULL, _IONBF, 0);
  setvbuf(stdout, NULL, _IONBF, 0);
  stdin->_flag &= ~_IOCOOKED;
}

void serial_init(void) {
  gcode_fd = stdin;
  makeraw();
  serial_initialised = true;
}

// return number of characters in the receive buffer
uint8_t serial_rxchars(void) {
  sim_assert(serial_initialised, "serial interface not initialised");
  return kbhit();
}

// read one character
uint8_t serial_popchar(void) {
  uint8_t c;
  ssize_t count;
  sim_assert(serial_initialised, "serial interface not initialised");
  sim_assert(serial_rxchars() > 0, "no chars to read");
  if ((count = kbhit()))
  {
    c = getchar();
    putchar(c);
  }    
  sim_assert(count == 1, "no character in serial RX buffer");
  return c;
}

// send one character
void serial_writechar(uint8_t data) {
  sim_assert(serial_initialised, "serial interface not initialised");
  sim_gcode_ch(data);
  if (serial_fd) {
    ssize_t count;
    count = write(serial_fd, &data, 1);
    sim_assert(count == 1, "could not write to serial port");
  }
}

// read/write many characters
void serial_writestr(uint8_t *data) {
  const char *str = (char *)data;
  sim_assert(serial_initialised, "serial interface not initialised");
  sim_gcode(str);
  if (serial_fd) {
    ssize_t count;
    count = write(serial_fd, str, strlen(str));
    sim_assert(count == strlen(str), "could not write to serial port");
  }
}

// write from flash
void serial_writestr_P(PGM_P data) {
  serial_writestr((uint8_t *)data);
}
